@@ -43,7 +43,7 @@ int main(void) {
     for (i = 0; i < n; i++) {
         for (j = i + 1; j < n; j++) {
             igraph_bool_t conn;
-            igraph_are_connected(&g, i, j, &conn);
+            igraph_are_adjacent(&g, i, j, &conn);
             if (conn) {
                 continue;
             }
