@@ -74,7 +74,7 @@ igraph_error_t igraph_clusters(const igraph_t *graph, igraph_vector_int_t *membe
  *        Alternatively this argument can be \c NULL, in which
  *        case it is ignored.
  * \param no Pointer to an integer, if not \c NULL then the number of
- *        clusters will be stored here.
+ *        components will be stored here.
  * \param mode For directed graph this specifies whether to calculate
  *        weakly or strongly connected components. Possible values:
  *        \c IGRAPH_WEAK,
@@ -99,7 +99,7 @@ igraph_error_t igraph_connected_components(
         return igraph_i_connected_components_strong(graph, membership, csize, no);
     }
 
-    IGRAPH_ERROR("Cannot calculate connected components.", IGRAPH_EINVAL);
+    IGRAPH_ERROR("Invalid connectedness mode.", IGRAPH_EINVAL);
 }
 
 static igraph_error_t igraph_i_connected_components_weak(
@@ -108,21 +108,11 @@ static igraph_error_t igraph_i_connected_components_weak(
 ) {
 
     igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_integer_t no_of_components;
     bool *already_added;
-    igraph_integer_t first_node, act_cluster_size = 0, no_of_clusters = 0;
-
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
-
-    igraph_integer_t i;
     igraph_vector_int_t neis = IGRAPH_VECTOR_NULL;
 
-    already_added = IGRAPH_CALLOC(no_of_nodes, bool);
-    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for calculating weakly connected components.");
-    IGRAPH_FINALLY(igraph_free, already_added);
-
-    IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, no_of_nodes > 100000 ? 10000 : no_of_nodes / 10);
-    IGRAPH_VECTOR_INT_INIT_FINALLY(&neis, 0);
-
     /* Memory for result, csize is dynamically allocated */
     if (membership) {
         IGRAPH_CHECK(igraph_vector_int_resize(membership, no_of_nodes));
@@ -131,18 +121,50 @@ static igraph_error_t igraph_i_connected_components_weak(
         igraph_vector_int_clear(csize);
     }
 
+    /* Try to make use of cached information. */
+    if (igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED) &&
+        igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED)) {
+        /* If we know that the graph is weakly connected from the cache,
+         * we can return the result right away. We keep in mind that
+         * the null graph is considered disconnected, therefore any connected
+         * graph has precisely one component. */
+        if (membership) {
+            /* All vertices are members of the same component. */
+            igraph_vector_int_fill(membership, 0);
+        }
+        if (csize) {
+            /* The size of the single component is the same as the vertex count. */
+            IGRAPH_CHECK(igraph_vector_int_push_back(csize, no_of_nodes));
+        }
+        if (no) {
+            /* There is one component. */
+            *no = 1;
+        }
+        return IGRAPH_SUCCESS;
+    }
+
+    already_added = IGRAPH_CALLOC(no_of_nodes, bool);
+    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for calculating weakly connected components.");
+    IGRAPH_FINALLY(igraph_free, already_added);
+
+    IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, no_of_nodes > 100000 ? 10000 : no_of_nodes / 10);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&neis, 0);
+
     /* The algorithm */
 
-    for (first_node = 0; first_node < no_of_nodes; ++first_node) {
+    no_of_components = 0;
+    for (igraph_integer_t first_node = 0; first_node < no_of_nodes; ++first_node) {
+        igraph_integer_t act_component_size;
+
         if (already_added[first_node]) {
             continue;
         }
         IGRAPH_ALLOW_INTERRUPTION();
 
         already_added[first_node] = true;
-        act_cluster_size = 1;
+        act_component_size = 1;
         if (membership) {
-            VECTOR(*membership)[first_node] = no_of_clusters;
+            VECTOR(*membership)[first_node] = no_of_components;
         }
         IGRAPH_CHECK(igraph_dqueue_int_push(&q, first_node));
 
@@ -150,30 +172,30 @@ static igraph_error_t igraph_i_connected_components_weak(
             igraph_integer_t act_node = igraph_dqueue_int_pop(&q);
             IGRAPH_CHECK(igraph_neighbors(graph, &neis, act_node, IGRAPH_ALL));
             igraph_integer_t nei_count = igraph_vector_int_size(&neis);
-            for (i = 0; i < nei_count; i++) {
+            for (igraph_integer_t i = 0; i < nei_count; i++) {
                 igraph_integer_t neighbor = VECTOR(neis)[i];
                 if (already_added[neighbor]) {
                     continue;
                 }
                 IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
                 already_added[neighbor] = true;
-                act_cluster_size++;
+                act_component_size++;
                 if (membership) {
-                    VECTOR(*membership)[neighbor] = no_of_clusters;
+                    VECTOR(*membership)[neighbor] = no_of_components;
                 }
             }
         }
 
-        no_of_clusters++;
+        no_of_components++;
         if (csize) {
-            IGRAPH_CHECK(igraph_vector_int_push_back(csize, act_cluster_size));
+            IGRAPH_CHECK(igraph_vector_int_push_back(csize, act_component_size));
         }
     }
 
     /* Cleaning up */
 
     if (no) {
-        *no = no_of_clusters;
+        *no = no_of_components;
     }
 
     /* Clean up */
@@ -183,7 +205,7 @@ static igraph_error_t igraph_i_connected_components_weak(
     IGRAPH_FINALLY_CLEAN(3);
 
     /* Update cache */
-    igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, no_of_clusters == 1);
+    igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, no_of_components == 1);
 
     return IGRAPH_SUCCESS;
 }
@@ -194,39 +216,57 @@ static igraph_error_t igraph_i_connected_components_strong(
 ) {
     igraph_integer_t no_of_nodes = igraph_vcount(graph);
     igraph_vector_int_t next_nei = IGRAPH_VECTOR_NULL;
-
-    igraph_integer_t i, n, num_seen;
+    igraph_integer_t num_seen;
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
-
-    igraph_integer_t no_of_clusters = 0;
-    igraph_integer_t act_cluster_size;
-
+    igraph_integer_t no_of_components = 0;
     igraph_vector_int_t out = IGRAPH_VECTOR_NULL;
-    const igraph_vector_int_t* tmp;
-
     igraph_adjlist_t adjlist;
 
+    /* Memory for result, csize is dynamically allocated */
+    if (membership) {
+        IGRAPH_CHECK(igraph_vector_int_resize(membership, no_of_nodes));
+    }
+    if (csize) {
+        igraph_vector_int_clear(csize);
+    }
+
+    /* Try to make use of cached information. */
+    if (igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_STRONGLY_CONNECTED) &&
+        igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_STRONGLY_CONNECTED)) {
+        /* If we know that the graph is strongly connected from the cache,
+         * we can return the result right away. We keep in mind that
+         * the null graph is considered disconnected, therefore any connected
+         * graph has precisely one component. */
+        if (membership) {
+            /* All vertices are members of the same component. */
+            igraph_vector_int_fill(membership, 0);
+        }
+        if (csize) {
+            /* The size of the single component is the same as the vertex count. */
+            IGRAPH_CHECK(igraph_vector_int_push_back(csize, no_of_nodes));
+        }
+        if (no) {
+            /* There is one component. */
+            *no = 1;
+        }
+        return IGRAPH_SUCCESS;
+    }
+
     /* The result */
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&next_nei, no_of_nodes);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&out, 0);
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, 100);
 
-    if (membership) {
-        IGRAPH_CHECK(igraph_vector_int_resize(membership, no_of_nodes));
-    }
     IGRAPH_CHECK(igraph_vector_int_reserve(&out, no_of_nodes));
 
-    igraph_vector_int_null(&out);
-    if (csize) {
-        igraph_vector_int_clear(csize);
-    }
-
     IGRAPH_CHECK(igraph_adjlist_init(graph, &adjlist, IGRAPH_OUT, IGRAPH_LOOPS_ONCE, IGRAPH_MULTIPLE));
     IGRAPH_FINALLY(igraph_adjlist_destroy, &adjlist);
 
     num_seen = 0;
-    for (i = 0; i < no_of_nodes; i++) {
+    for (igraph_integer_t i = 0; i < no_of_nodes; i++) {
+        const igraph_vector_int_t *tmp;
+
         IGRAPH_ALLOW_INTERRUPTION();
 
         tmp = igraph_adjlist_get(&adjlist, i);
@@ -279,15 +319,16 @@ static igraph_error_t igraph_i_connected_components_strong(
     num_seen = 0;
 
     while (!igraph_vector_int_empty(&out)) {
+        igraph_integer_t act_component_size;
         igraph_integer_t grandfather = igraph_vector_int_pop_back(&out);
 
         if (VECTOR(next_nei)[grandfather] != 0) {
             continue;
         }
         VECTOR(next_nei)[grandfather] = 1;
-        act_cluster_size = 1;
+        act_component_size = 1;
         if (membership) {
-            VECTOR(*membership)[grandfather] = no_of_clusters;
+            VECTOR(*membership)[grandfather] = no_of_components;
         }
         IGRAPH_CHECK(igraph_dqueue_int_push(&q, grandfather));
 
@@ -301,18 +342,18 @@ static igraph_error_t igraph_i_connected_components_strong(
 
         while (!igraph_dqueue_int_empty(&q)) {
             igraph_integer_t act_node = igraph_dqueue_int_pop_back(&q);
-            tmp = igraph_adjlist_get(&adjlist, act_node);
-            n = igraph_vector_int_size(tmp);
-            for (i = 0; i < n; i++) {
+            const igraph_vector_int_t *tmp = igraph_adjlist_get(&adjlist, act_node);
+            const igraph_integer_t n = igraph_vector_int_size(tmp);
+            for (igraph_integer_t i = 0; i < n; i++) {
                 igraph_integer_t neighbor = VECTOR(*tmp)[i];
                 if (VECTOR(next_nei)[neighbor] != 0) {
                     continue;
                 }
                 IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
                 VECTOR(next_nei)[neighbor] = 1;
-                act_cluster_size++;
+                act_component_size++;
                 if (membership) {
-                    VECTOR(*membership)[neighbor] = no_of_clusters;
+                    VECTOR(*membership)[neighbor] = no_of_components;
                 }
 
                 num_seen++;
@@ -325,16 +366,16 @@ static igraph_error_t igraph_i_connected_components_strong(
             }
         }
 
-        no_of_clusters++;
+        no_of_components++;
         if (csize) {
-            IGRAPH_CHECK(igraph_vector_int_push_back(csize, act_cluster_size));
+            IGRAPH_CHECK(igraph_vector_int_push_back(csize, act_component_size));
         }
     }
 
     IGRAPH_PROGRESS("Strongly connected components: ", 100.0, NULL);
 
     if (no) {
-        *no = no_of_clusters;
+        *no = no_of_components;
     }
 
     /* Clean up */
@@ -345,15 +386,15 @@ static igraph_error_t igraph_i_connected_components_strong(
     IGRAPH_FINALLY_CLEAN(4);
 
     /* Update cache */
-    igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_STRONGLY_CONNECTED, no_of_clusters == 1);
-    if (no_of_clusters == 1) {
-        igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, 1);
+    igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_STRONGLY_CONNECTED, no_of_components == 1);
+    if (no_of_components == 1) {
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, true);
     }
 
     return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t igraph_is_connected_weak(const igraph_t *graph, igraph_bool_t *res);
+static igraph_error_t igraph_i_is_connected_weak(const igraph_t *graph, igraph_bool_t *res);
 
 /**
  * \ingroup structural
@@ -427,7 +468,7 @@ igraph_error_t igraph_is_connected(const igraph_t *graph, igraph_bool_t *res,
     } else if (no_of_nodes == 1) {
         *res = true;
     } else if (mode == IGRAPH_WEAK) {
-        IGRAPH_CHECK(igraph_is_connected_weak(graph, res));
+        IGRAPH_CHECK(igraph_i_is_connected_weak(graph, res));
     } else {   /* mode == IGRAPH_STRONG */
         /* A strongly connected graph has at least as many edges as vertices,
          * except for the singleton graph, which is handled above. */
@@ -440,16 +481,16 @@ igraph_error_t igraph_is_connected(const igraph_t *graph, igraph_bool_t *res,
     }
 
     /* Cache updates are done in igraph_i_connected_components_strong() and
-     * igraph_is_connected_weak() because those might be called from other
+     * igraph_i_is_connected_weak() because those might be called from other
      * places and we want to make use of the caching if so */
 
     return IGRAPH_SUCCESS;
 }
 
-static igraph_error_t igraph_is_connected_weak(const igraph_t *graph, igraph_bool_t *res) {
+static igraph_error_t igraph_i_is_connected_weak(const igraph_t *graph, igraph_bool_t *res) {
     igraph_integer_t no_of_nodes = igraph_vcount(graph), no_of_edges = igraph_ecount(graph);
     igraph_integer_t added_count;
-    char *already_added;
+    bool *already_added;
     igraph_vector_int_t neis = IGRAPH_VECTOR_NULL;
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
 
@@ -466,17 +507,15 @@ static igraph_error_t igraph_is_connected_weak(const igraph_t *graph, igraph_boo
         goto exit;
     }
 
-    already_added = IGRAPH_CALLOC(no_of_nodes, char);
-    if (already_added == 0) {
-        IGRAPH_ERROR("Weak connectedness check failed.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    already_added = IGRAPH_CALLOC(no_of_nodes, bool);
+    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for computing weakly connected components.");
     IGRAPH_FINALLY(igraph_free, already_added);
 
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, 10);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&neis, 0);
 
-    /* Try to find at least two clusters */
-    already_added[0] = 1;
+    /* Try to find at least two components */
+    already_added[0] = true;
     IGRAPH_CHECK(igraph_dqueue_int_push(&q, 0));
 
     added_count = 1;
@@ -496,7 +535,7 @@ static igraph_error_t igraph_is_connected_weak(const igraph_t *graph, igraph_boo
 
             IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
             added_count++;
-            already_added[neighbor] = 1;
+            already_added[neighbor] = true;
 
             if (added_count == no_of_nodes) {
                 /* We have already reached all nodes: the graph is connected.
@@ -516,11 +555,11 @@ static igraph_error_t igraph_is_connected_weak(const igraph_t *graph, igraph_boo
     IGRAPH_FINALLY_CLEAN(3);
 
 exit:
-    igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, *res);
+    igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, *res);
     if (igraph_is_directed(graph) && *res == 0) {
         /* If the graph is not weakly connected, it is not strongly connected
          * either so we can also cache that */
-        igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_STRONGLY_CONNECTED, *res);
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_STRONGLY_CONNECTED, *res);
     }
 
     return IGRAPH_SUCCESS;
@@ -614,7 +653,7 @@ static igraph_error_t igraph_i_decompose_weak(const igraph_t *graph,
     igraph_integer_t actstart;
     igraph_integer_t no_of_nodes = igraph_vcount(graph);
     igraph_integer_t resco = 0;   /* number of graphs created so far */
-    char *already_added;
+    bool *already_added;
     igraph_dqueue_int_t q;
     igraph_vector_int_t verts;
     igraph_vector_int_t neis;
@@ -630,10 +669,8 @@ static igraph_error_t igraph_i_decompose_weak(const igraph_t *graph,
     igraph_graph_list_clear(components);
 
     /* already_added keeps track of what nodes made it into a graph already */
-    already_added = IGRAPH_CALLOC(no_of_nodes, char);
-    if (already_added == 0) {
-        IGRAPH_ERROR("Cannot decompose graph", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    already_added = IGRAPH_CALLOC(no_of_nodes, bool);
+    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for decomponsing graph into connected components.");
     IGRAPH_FINALLY(igraph_free, already_added);
 
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&q, 100);
@@ -657,7 +694,7 @@ static igraph_error_t igraph_i_decompose_weak(const igraph_t *graph,
         igraph_vector_int_clear(&verts);
 
         /* add the node itself */
-        already_added[actstart] = 1;
+        already_added[actstart] = true;
         IGRAPH_CHECK(igraph_vector_int_push_back(&verts, actstart));
         IGRAPH_CHECK(igraph_dqueue_int_push(&q, actstart));
 
@@ -670,11 +707,11 @@ static igraph_error_t igraph_i_decompose_weak(const igraph_t *graph,
             /* iterate over the neighbors */
             for (i = 0; i < nei_count; i++) {
                 igraph_integer_t neighbor = VECTOR(neis)[i];
-                if (already_added[neighbor] == 1) {
+                if (already_added[neighbor]) {
                     continue;
                 }
                 /* add neighbor */
-                already_added[neighbor] = 1;
+                already_added[neighbor] = true;
 
                 /* recursion: append neighbor to the queues */
                 IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
@@ -728,7 +765,7 @@ static igraph_error_t igraph_i_decompose_strong(const igraph_t *graph,
     igraph_integer_t i, n, num_seen;
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
 
-    igraph_integer_t no_of_clusters = 0;
+    igraph_integer_t no_of_components = 0;
 
     igraph_vector_int_t out = IGRAPH_VECTOR_NULL;
     const igraph_vector_int_t* tmp;
@@ -835,7 +872,7 @@ static igraph_error_t igraph_i_decompose_strong(const igraph_t *graph,
 
     /* number of components built */
     num_seen = 0;
-    while (!igraph_vector_int_empty(&out) && no_of_clusters < maxcompno) {
+    while (!igraph_vector_int_empty(&out) && no_of_components < maxcompno) {
         /* consume the vector from the last element */
         igraph_integer_t grandfather = igraph_vector_int_pop_back(&out);
 
@@ -913,7 +950,7 @@ static igraph_error_t igraph_i_decompose_strong(const igraph_t *graph,
             VECTOR(vids_old2new)[VECTOR(verts)[i]] = 0;
         }
 
-        no_of_clusters++;
+        no_of_components++;
     }
 
     IGRAPH_PROGRESS("Strongly connected components: ", 100.0, NULL);
@@ -937,21 +974,28 @@ static igraph_error_t igraph_i_decompose_strong(const igraph_t *graph,
  * \brief Finds the articulation points in a graph.
  *
  * A vertex is an articulation point if its removal increases
- * the number of connected components in the graph.
+ * the number of (weakly) connected components in the graph.
  *
- * \param graph The input graph.
- * \param res Pointer to an initialized vector, the
- *    articulation points will be stored here.
+ * </para><para>
+ * Note that a graph without any articulation points is not necessarily
+ * biconnected. Counterexamples are the two-vertex complete graph as well
+ * as empty graphs. Use \ref igraph_is_biconnected() to check whether
+ * a graph is biconnected.
+ *
+ * \param graph The input graph. It will be treated as undirected.
+ * \param res Pointer to an initialized vector, the articulation points will
+ *   be stored here.
  * \return Error code.
  *
  * Time complexity: O(|V|+|E|), linear in the number of vertices and edges.
  *
- * \sa \ref igraph_biconnected_components(), \ref igraph_clusters(), \ref igraph_bridges()
+ * \sa \ref igraph_biconnected_components(), \ref igraph_is_bipartite(),
+ * \ref igraph_connected_components(), \ref igraph_bridges()
  */
 
 igraph_error_t igraph_articulation_points(const igraph_t *graph, igraph_vector_int_t *res) {
 
-    return igraph_biconnected_components(graph, 0, 0, 0, 0, res);
+    return igraph_biconnected_components(graph, NULL, NULL, NULL, NULL, res);
 }
 
 /**
@@ -964,21 +1008,23 @@ igraph_error_t igraph_articulation_points(const igraph_t *graph, igraph_vector_i
  * </para><para>
  * A biconnected component of a graph is a maximal biconnected
  * subgraph of it. The biconnected components of a graph can be given
- * by the partition of its edges: every edge is a member of exactly
+ * by a partition of its edges: every edge is a member of exactly
  * one biconnected component. Note that this is not true for
  * vertices: the same vertex can be part of many biconnected
- * components.
+ * components, while isolated vertices are part of none at all.
  *
  * </para><para>
  * Note that some authors do not consider the graph consisting of
  * two connected vertices as biconnected, however, igraph does.
  *
  * </para><para>
- * Somewhat arbitrarily, igraph does not consider components containing
- * a single vertex only as being biconnected. Isolated vertices will
- * not be part of any of the biconnected components.
+ * igraph does not consider components containing a single vertex only as
+ * being biconnected. Isolated vertices will not be part of any of the
+ * biconnected components. This means that checking whether there is a single
+ * biconnected component is not sufficient for determining if a graph is
+ * biconnected. Use \ref igraph_is_biconnected() for this purpose.
  *
- * \param graph The input graph.
+ * \param graph The input graph. It will be treated as undirected.
  * \param no If not a NULL pointer, the number of biconnected components will
  *     be stored here.
  * \param tree_edges If not a NULL pointer, then the found components
@@ -998,13 +1044,14 @@ igraph_error_t igraph_articulation_points(const igraph_t *graph, igraph_vector_i
  * \return Error code.
  *
  * Time complexity: O(|V|+|E|), linear in the number of vertices and
- * edges, but only if you do not calculate \c components and
- * \c component_edges. If you calculate \c components, then it is
- * quadratic in the number of vertices. If you calculate \c
- * component_edges as well, then it is cubic in the number of
+ * edges, but only if you do not calculate \p components and
+ * \p component_edges. If you calculate \p components, then it is
+ * quadratic in the number of vertices. If you calculate
+ * \p component_edges as well, then it is cubic in the number of
  * vertices.
  *
- * \sa \ref igraph_articulation_points(), \ref igraph_clusters().
+ * \sa \ref igraph_articulation_points(), \ref igraph_is_biconnected(),
+ * \ref igraph_connected_components().
  *
  * \example examples/simple/igraph_biconnected_components.c
  */
@@ -1113,7 +1160,7 @@ igraph_error_t igraph_biconnected_components(const igraph_t *graph,
                         if (articulation_points && !VECTOR(found)[prev]
                             && prev != i /* the root */) {
                             IGRAPH_CHECK(igraph_vector_int_push_back(articulation_points, prev));
-                            VECTOR(found)[prev] = 1;
+                            VECTOR(found)[prev] = true;
                         }
                         if (no) {
                             *no += 1;
@@ -1206,6 +1253,160 @@ igraph_error_t igraph_biconnected_components(const igraph_t *graph,
     return IGRAPH_SUCCESS;
 }
 
+/**
+ * \function igraph_is_biconnected
+ * \brief Checks whether a graph is biconnected.
+ *
+ * \experimental
+ *
+ * A graph is biconnected if the removal of any single vertex (and
+ * its incident edges) does not disconnect it.
+ *
+ * </para><para>
+ * igraph does not consider single-vertex graphs biconnected.
+ *
+ * </para><para>
+ * Note that some authors do not consider the graph consisting of
+ * two connected vertices as biconnected, however, igraph does.
+ *
+ * \param graph The input graph. It will be treated as undirected.
+ * \param result If not a \c NULL pointer, the result will be returned here.
+ * \return Error code.
+ *
+ * Time complexity: O(|V|+|E|), linear in the number of vertices and edges.
+ *
+ * \sa \ref igraph_articulation_points(), \ref igraph_biconnected_components().
+ *
+ * \example examples/simple/igraph_is_biconnected.c
+ */
+
+igraph_error_t igraph_is_biconnected(const igraph_t *graph, igraph_bool_t *res) {
+
+    igraph_integer_t no_of_nodes = igraph_vcount(graph);
+    igraph_vector_int_t nextptr;
+    igraph_vector_int_t num, low;
+    igraph_stack_int_t path;
+    igraph_lazy_adjlist_t inclist;
+    igraph_bool_t is_biconnected = true;
+
+    if (no_of_nodes == 0 || no_of_nodes == 1) {
+        /* The null graph is not connected, hence it is not biconnected either.
+         * The singleton graph is not biconnected. */
+        is_biconnected = false;
+        goto exit2;
+    }
+
+    /* no_of_nodes == 2 is special: if the two nodes are connected, then the
+     * graph is both biconnected _and_ acyclic, unlike no_of_nodes >= 3, where
+     * the graph is not acyclic if it is biconnected. */
+
+    /* We do not touch the cache for graphs with less than three nodes because
+     * of all the edge cases. */
+    if (no_of_nodes >= 3 && (
+        (igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED) &&
+        !igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED)) ||
+        (igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_FOREST) &&
+        igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_FOREST))
+    )) {
+        is_biconnected = false;
+        goto exit2;
+    }
+
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&nextptr, no_of_nodes);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&num, no_of_nodes);
+    IGRAPH_VECTOR_INT_INIT_FINALLY(&low, no_of_nodes);
+
+    IGRAPH_STACK_INT_INIT_FINALLY(&path, 100);
+
+    IGRAPH_CHECK(igraph_lazy_adjlist_init(graph, &inclist, IGRAPH_ALL, IGRAPH_NO_LOOPS, IGRAPH_NO_MULTIPLE));
+    IGRAPH_FINALLY(igraph_lazy_adjlist_destroy, &inclist);
+
+    const igraph_integer_t root = 0; /* start DFS from vertex 0 */
+    igraph_integer_t counter = 1;
+    igraph_integer_t rootdfs = 0;
+    IGRAPH_CHECK(igraph_stack_int_push(&path, root));
+    VECTOR(low)[root] = VECTOR(num)[root] = counter++;
+    while (!igraph_stack_int_empty(&path)) {
+        igraph_integer_t act = igraph_stack_int_top(&path);
+        igraph_integer_t actnext = VECTOR(nextptr)[act];
+
+        const igraph_vector_int_t *neis = igraph_lazy_adjlist_get(&inclist, act);
+        const igraph_integer_t n = igraph_vector_int_size(neis);
+        if (actnext < n) {
+            /* Step down (maybe) */
+            igraph_integer_t nei = VECTOR(*neis)[actnext];
+            if (VECTOR(low)[nei] == 0) {
+                if (act == root) {
+                    rootdfs++;
+                }
+                IGRAPH_CHECK(igraph_stack_int_push(&path, nei));
+                VECTOR(low)[nei] = VECTOR(num)[nei] = counter++;
+            } else {
+                /* Update low value if needed */
+                if (VECTOR(num)[nei] < VECTOR(low)[act]) {
+                    VECTOR(low)[act] = VECTOR(num)[nei];
+                }
+            }
+            VECTOR(nextptr)[act] += 1;
+        } else {
+            /* Step up */
+            igraph_stack_int_pop(&path);
+            if (!igraph_stack_int_empty(&path)) {
+                igraph_integer_t prev = igraph_stack_int_top(&path);
+                /* Update LOW value if needed */
+                if (VECTOR(low)[act] < VECTOR(low)[prev]) {
+                    VECTOR(low)[prev] = VECTOR(low)[act];
+                }
+                /* Check for articulation point */
+                if (VECTOR(low)[act] >= VECTOR(num)[prev]) {
+                    if (prev != root /* the root */) {
+                        /* Found an articulation point, the graph is not biconnected */
+                        is_biconnected = false;
+                        goto exit;
+                    }
+                }
+            } /* !igraph_stack_int_empty(&path) */
+        }
+
+    } /* !igraph_stack_int_empty(&path) */
+
+    /* The root is an articulation point, the graph is not biconnected */
+    if (rootdfs >= 2) {
+        is_biconnected = false;
+        goto exit;
+    }
+
+    /* We did not reach all vertices, the graph is not connected */
+    if (counter <= no_of_nodes) {
+        is_biconnected = false;
+        goto exit;
+    }
+
+exit:
+
+    igraph_lazy_adjlist_destroy(&inclist);
+    igraph_stack_int_destroy(&path);
+    igraph_vector_int_destroy(&low);
+    igraph_vector_int_destroy(&num);
+    igraph_vector_int_destroy(&nextptr);
+    IGRAPH_FINALLY_CLEAN(5);
+
+exit2:
+
+    if (res) {
+        *res = is_biconnected;
+    }
+
+    /* We do not touch the cache for graphs with less than three nodes because
+     * of all the edge cases. */
+    if (is_biconnected && no_of_nodes >= 3) {
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, true);
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_FOREST, false);
+    }
+
+    return IGRAPH_SUCCESS;
+}
+
 
 /**
  * \function igraph_bridges
@@ -1221,7 +1422,8 @@ igraph_error_t igraph_biconnected_components(const igraph_t *graph,
  *
  * Time complexity: O(|V|+|E|), linear in the number of vertices and edges.
  *
- * \sa \ref igraph_articulation_points(), \ref igraph_biconnected_components(), \ref igraph_clusters()
+ * \sa \ref igraph_articulation_points(), \ref igraph_biconnected_components(),
+ * \ref igraph_connected_components()
  */
 
 igraph_error_t igraph_bridges(const igraph_t *graph, igraph_vector_int_t *bridges) {
@@ -1275,7 +1477,7 @@ igraph_error_t igraph_bridges(const igraph_t *graph, igraph_vector_int_t *bridge
                 if (i == 0) {
                     /* We are at the first step of visiting vertex u. */
 
-                    VECTOR(visited)[u] = 1;
+                    VECTOR(visited)[u] = true;
 
                     time += 1;
 
@@ -1379,7 +1581,7 @@ igraph_error_t igraph_subcomponent(
 
     igraph_integer_t no_of_nodes = igraph_vcount(graph);
     igraph_dqueue_int_t q = IGRAPH_DQUEUE_NULL;
-    char *already_added;
+    bool *already_added;
     igraph_integer_t i, vsize;
     igraph_vector_int_t tmp = IGRAPH_VECTOR_NULL;
 
@@ -1391,10 +1593,8 @@ igraph_error_t igraph_subcomponent(
         IGRAPH_ERROR("Invalid mode argument.", IGRAPH_EINVMODE);
     }
 
-    already_added = IGRAPH_CALLOC(no_of_nodes, char);
-    if (already_added == 0) {
-        IGRAPH_ERROR("Subcomponent failed.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-    }
+    already_added = IGRAPH_CALLOC(no_of_nodes, bool);
+    IGRAPH_CHECK_OOM(already_added, "Insufficient memory for computing subcomponent.");
     IGRAPH_FINALLY(igraph_free, already_added);
 
     igraph_vector_int_clear(res);
@@ -1404,7 +1604,7 @@ igraph_error_t igraph_subcomponent(
 
     IGRAPH_CHECK(igraph_dqueue_int_push(&q, vertex));
     IGRAPH_CHECK(igraph_vector_int_push_back(res, vertex));
-    already_added[vertex] = 1;
+    already_added[vertex] = true;
 
     while (!igraph_dqueue_int_empty(&q)) {
         igraph_integer_t actnode = igraph_dqueue_int_pop(&q);
@@ -1419,7 +1619,7 @@ igraph_error_t igraph_subcomponent(
             if (already_added[neighbor]) {
                 continue;
             }
-            already_added[neighbor] = 1;
+            already_added[neighbor] = true;
             IGRAPH_CHECK(igraph_vector_int_push_back(res, neighbor));
             IGRAPH_CHECK(igraph_dqueue_int_push(&q, neighbor));
         }
