@@ -73,20 +73,6 @@ igraph_error_t igraph_strvector_init(igraph_strvector_t *sv, igraph_integer_t si
     sv->stor_begin = IGRAPH_CALLOC(size, char*);
     IGRAPH_CHECK_OOM(sv->stor_begin, "Cannot initialize string vector.");
 
-    for (igraph_integer_t i = 0; i < size; i++) {
-        sv->stor_begin[i] = IGRAPH_CALLOC(1, char);
-        if (sv->stor_begin[i] == NULL) {
-            /* LCOV_EXCL_START */
-            for (igraph_integer_t j = 0; j < i; j++) {
-                IGRAPH_FREE(sv->stor_begin[j]);
-            }
-            IGRAPH_FREE(sv->stor_begin);
-            IGRAPH_ERROR("Cannot initialize string vector.", IGRAPH_ENOMEM); /* LCOV_EXCL_LINE */
-            /* LCOV_EXCL_STOP */
-        }
-        sv->stor_begin[i][0] = '\0';
-    }
-
     sv->stor_end = sv->stor_begin + size;
     sv->end = sv->stor_end;
 
@@ -119,10 +105,9 @@ void igraph_strvector_destroy(igraph_strvector_t *sv) {
 /**
  * \ingroup strvector
  * \function igraph_strvector_get
- * \brief Retrieves an element of the string vector.
+ * \brief Retrieves an element of a string vector.
  *
- * Query an element of a string vector. See also the \ref STR macro
- * for an easier way.
+ * Query an element of a string vector. The returned string must not be modified.
  *
  * \param sv The input string vector.
  * \param idx The index of the element to query.
@@ -133,8 +118,7 @@ void igraph_strvector_destroy(igraph_strvector_t *sv) {
 const char *igraph_strvector_get(const igraph_strvector_t *sv, igraph_integer_t idx) {
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
-    IGRAPH_ASSERT(sv->stor_begin[idx] != NULL);
-    return sv->stor_begin[idx];
+    return sv->stor_begin[idx] ? sv->stor_begin[idx] : "";
 }
 
 /**
@@ -178,18 +162,20 @@ igraph_error_t igraph_strvector_set(igraph_strvector_t *sv, igraph_integer_t idx
  */
 igraph_error_t igraph_strvector_set_len(igraph_strvector_t *sv, igraph_integer_t idx,
                           const char *value, size_t len) {
-    char *tmp;
-
     IGRAPH_ASSERT(sv != NULL);
     IGRAPH_ASSERT(sv->stor_begin != NULL);
-    IGRAPH_ASSERT(sv->stor_begin[idx] != NULL);
 
-    tmp = IGRAPH_REALLOC(sv->stor_begin[idx], len + 1, char);
-    IGRAPH_CHECK_OOM(tmp, "Cannot reserve space for new item in string vector.");
+    if (sv->stor_begin[idx] == NULL) {
+        sv->stor_begin[idx] = strndup(value, len);
+        IGRAPH_CHECK_OOM(sv->stor_begin[idx], "Cannot reserve space for new item in string vector.");
+    } else {
+        char *tmp = IGRAPH_REALLOC(sv->stor_begin[idx], len + 1, char);
+        IGRAPH_CHECK_OOM(tmp, "Cannot reserve space for new item in string vector.");
 
-    sv->stor_begin[idx] = tmp;
-    memcpy(sv->stor_begin[idx], value, len * sizeof(char));
-    sv->stor_begin[idx][len] = '\0';
+        sv->stor_begin[idx] = tmp;
+        memcpy(sv->stor_begin[idx], value, len * sizeof(char));
+        sv->stor_begin[idx][len] = '\0';
+    }
 
     return IGRAPH_SUCCESS;
 }
@@ -268,7 +254,12 @@ igraph_error_t igraph_strvector_init_copy(igraph_strvector_t *to,
     IGRAPH_CHECK_OOM(to->stor_begin, "Cannot copy string vector.");
 
     for (igraph_integer_t i = 0; i < from_size; i++) {
-        to->stor_begin[i] = strdup(igraph_strvector_get(from, i));
+        /* If the string in the 'from' vector is empty, we represent it as NULL.
+         * The NULL value was already set by IGRAPH_CALLOC(). */
+        if (from->stor_begin[i] == NULL || from->stor_begin[i][0] == '\0') {
+            continue;
+        }
+        to->stor_begin[i] = strdup(from->stor_begin[i]);
         if (to->stor_begin[i] == NULL) {
             /* LCOV_EXCL_START */
             for (igraph_integer_t j = 0; j < i; j++) {
@@ -329,14 +320,18 @@ igraph_error_t igraph_strvector_append(igraph_strvector_t *to,
     IGRAPH_CHECK(igraph_strvector_reserve(to, newlen));
 
     for (igraph_integer_t i = 0; i < len2; i++) {
-        tmp = strdup(igraph_strvector_get(from, i));
-        if (!tmp) {
-            error = true;
-            break;
+        if (from->stor_begin[i] == NULL || from->stor_begin[i][0] == '\0') {
+            /* Represent empty strings as NULL. */
+            tmp = NULL;
         } else {
-            *(to->end) = tmp;
-            to->end++;
+            tmp = strdup(from->stor_begin[i]);
+            if (tmp == NULL) {
+                error = true;
+                break;
+            }
         }
+        *(to->end) = tmp;
+        to->end++;
     }
 
     if (error) {
@@ -436,19 +431,7 @@ igraph_error_t igraph_strvector_resize(igraph_strvector_t *sv, igraph_integer_t
         sv->end = sv->stor_begin + newsize;
     } else if (newsize > oldsize) {
         IGRAPH_CHECK(igraph_strvector_reserve(sv, newsize));
-
-        for (igraph_integer_t i = 0; i < toadd; i++) {
-            sv->stor_begin[oldsize + i] = IGRAPH_CALLOC(1, char);
-            if (sv->stor_begin[oldsize + i] == NULL) {
-                /* LCOV_EXCL_START */
-                for (igraph_integer_t j = 0; j < i; j++) {
-                    IGRAPH_FREE(sv->stor_begin[oldsize + j]);
-                }
-                IGRAPH_ERROR("Cannot resize string vector.", IGRAPH_ENOMEM);
-                /* LCOV_EXCL_STOP */
-            }
-            sv->stor_begin[oldsize + i][0] = '\0';
-        }
+        memset(sv->stor_begin + oldsize, 0, toadd * sizeof(char *));
         sv->end = sv->stor_begin + newsize;
     }
 
@@ -681,10 +664,10 @@ igraph_error_t igraph_strvector_print(const igraph_strvector_t *sv, FILE *file,
 
     igraph_integer_t n = igraph_strvector_size(sv);
     if (n != 0) {
-        fprintf(file, "%s", STR(*sv, 0));
+        fprintf(file, "%s", igraph_strvector_get(sv, 0));
     }
     for (igraph_integer_t i = 1; i < n; i++) {
-        fprintf(file, "%s%s", sep, STR(*sv, i));
+        fprintf(file, "%s%s", sep, igraph_strvector_get(sv, i));
     }
     return IGRAPH_SUCCESS;
 }
