@@ -1,9 +1,6 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph R library.
-   Copyright (C) 2003-2013  Gabor Csardi <csardi.gabor@gmail.com>
-   334 Harvard street, Cambridge, MA 02139 USA
+   IGraph library.
+   Copyright (C) 2003-2023  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,17 +13,15 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301 USA
-
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
-#include "igraph_vector.h"
+#include "igraph_games.h"
+
+#include "igraph_constructors.h"
 #include "igraph_matrix.h"
 #include "igraph_random.h"
-#include "igraph_constructors.h"
-#include "igraph_games.h"
+#include "igraph_vector.h"
 
 #include "core/interruption.h"
 #include "math/safe_intop.h"
@@ -67,7 +62,7 @@
  * vertices, |E| is the number of edges, and K is the number of
  * groups.
  *
- * \sa \ref igraph_erdos_renyi_game() for a simple Bernoulli graph.
+ * \sa \ref igraph_erdos_renyi_game_gnp() for a simple Bernoulli graph.
  *
  */
 
